#!/usr/bin/perl

########################################################################
#
# maemo training material generator version 0.1.2
#
# This maemo training material generator is licensed under MIT License.
# Copyright (c) 2007 Nokia Corporation. All rights reserved.
# 
########################################################################

# Prepend the dir of this script to the search paths so that the
# modified textile can be found.
use FindBin;
use lib "$FindBin::Bin";

use Text::Textile qw(textile);
use Getopt::Std;

getopts('o:i:n:');

if (!$opt_o || !$opt_i || !$opt_n) {
    die ("\nUsage:\n  $0 -o output_folder -i input_folder -n \"Document name\"\n\nAll parameters are required.\n\n");
}

my $outpath = File::Spec->rel2abs($opt_o);
my $inpath = File::Spec->rel2abs($opt_i);

print "Generating $opt_n...\n";

undef $/;
open INPUT, "<${inpath}/document.txt" or die ("Can't open input-file ${inpath}/document.txt\n");
$content = <INPUT>;
close INPUT;
open INPUT, "<${inpath}/header.txt" or die ("Can't open input-file ${inpath}/header.txt\n");
$header = <INPUT>;
close INPUT;
open INPUT, "<${inpath}/footer.txt" or die ("Can't open input-file ${inpath}/header.txt\n");
$footer = <INPUT>;
close INPUT;
open INPUT, "<${inpath}/license.txt" or die ("Can't open input-file ${inpath}/license.txt\n");
$license = <INPUT>;
close INPUT;
$/ = "\n";     # Restore normal behaviour

my $topnavi = "<ul>\n";

while ($content =~ /\[:CHAPTERSTART=?\"?(.*?)\"?:\](.*?)\[:CHAPTEREND:\]/gs) {
    my $filename = $1;
    my $title = $filename;
    my $data = $2;
    print "  $filename\n";
    $filename =~ s/\ /_/g;            # These five lines
    $filename =~ s/\?//g;             # sanitize filename to contain only
    $filename =~ s/\-//g;             # allowed characters which are:
    $filename =~ s/__/_/g;            # a-z A-Z 0-9 _ and -
    $filename =~ s/[^-a-zA-Z0-9_]//g; ####################################
    $title =~ s/^\s+//;
  
    my $navicount = 0;
    my $navi = "";
    my $topnaviplus = "";

    while ($data =~ /^(h([12])\.\s.*)/m) {
	my $orig = $1;
	my $match = $orig;
	my $type = $2;
	$match =~ s/h[12]\.\s//g;
	$orig =~ s/h[12]\.\s//g;
	$orig =~ s/://g;
	$match =~ s/[[:^alnum:]]//g;
	my $replace = "<a name=\"$match\"></a>\n<h$type>$orig</h$type>";
	$data =~ s/^(h[12]\.\s.*)/$replace/m;
	$navi .= "<li><a href=\"#$match\">$orig</a>\n";
	$topnaviplus .= "<li>$orig\n";
	$navicount++;
    }

    $navi = "<h2>Chapter contents:</h2>\n<p><ul>\n" . $navi . "</ul></p>\n";
    $topnavi .= "<li><a href=\"${filename}.html\"><b>${title}</b></a>\n<ul>\n" . $topnaviplus . "\n</ul>\n";

    #
    # This fixes some problems with tags that Textile.pm can't
    # handle correctly.
    # This isn't good either, and the fix Should actually be 
    # made into Textile.pm!
    #
    $data =~ s/@(.*?[^\\])@/\<code\>$1\<\/code\>/gm;
    $data =~ s/\_(.*?[^\\])\_/\<em\>$1\<\/em\>/gm;
    $data =~ s/\*\*(.*?[^\\])\*\*/\<b\>$1\<\/b\>/gm;
    $data =~ s/\\([\@\_\*])/$1/gm;

    my $html = textile ($data);

    #
    # Un-Textile some things
    #
    $html =~ s/\<\!--/\&lt;--/g;         # enable comments in pre 
                                         # (for xml)
    $html =~ s/\&lt;b\&gt;/\<b\>/g;      # enable <b> in <pre>
    $html =~ s/\&lt;\/b\&gt;/\<\/b\>/g;  # enable </b> in <pre>

    while ($html =~ /(\[:CODE=\"(.*?)\"(\s?(LANG)?=?\"?(.*?)\"?):\])/) {
        my $lang = "";
        if ($4 eq "LANG") {
            $lang = " --src-lang=" . $5 . " ";
        }
        my $replace = `source-highlight -f xhtml --input=$inpath/$2 $lang --failsafe --output=STDOUT`;
        $html =~ s/\[:CODE=\"$2\"$3:\]/$replace/g;
    }

    my $tempheader = $header;
    $tempheader =~ s/\[\:TITLE\:\]/$title/;
    open (NEWFILE, ">${outpath}/${filename}.html") or die ("Could not open file ${outpath}/${filename}.html for output");
    print NEWFILE "$tempheader";
    if ($navicount > 1) { 
	print NEWFILE "$navi";
    }
    print NEWFILE "$html";
    print NEWFILE "$footer";
    close (NEWFILE);
}

$topnavi .= "</ul>\n";
open (NEWFILE, ">${outpath}/index.html") or die ("Could not open file ${outpath}/index.html for output");
$header =~ s/\[\:TITLE\:\]/$opt_n/;
$license = textile ($license);
print NEWFILE $header;
print NEWFILE "<h1>${opt_n}</h1>\n\n";
print NEWFILE $license;
print NEWFILE "<h1>Table of Contents:</h1>\n\n";
print NEWFILE $topnavi;
print NEWFILE $footer;
close (NEWFILE);

print "$opt_n generated.\n\n";

